package com.soma.plugins.assets.tests {

	import com.soma.core.Soma;
	import com.soma.core.interfaces.ISoma;
	import com.soma.plugins.assets.SomaAssets;
	import com.soma.plugins.assets.events.SomaAssetsEvent;
	import com.soma.plugins.assets.vo.SomaAssetsVO;

	import org.assetloader.core.IAssetLoader;
	import org.flexunit.asserts.assertNotNull;
	import org.flexunit.asserts.fail;
	import org.flexunit.async.Async;

	import mx.core.FlexGlobals;

	import flash.display.Stage;
	import flash.net.URLRequest;
	/**
	 * <b>Author:</b> Romuald Quantin - <a href="http://www.soundstep.com/" target="_blank">www.soundstep.com</a><br />
	 * <b>Class version:</b> 1.0<br />
	 * <b>Actionscript version:</b> 3.0<br />
	 * <b>Date:</b> Oct 6, 2010<br />
	 * @example
	 * <listing version="3.0"></listing>
	 */
	public class EventsTests {


		private var _soma:ISoma;
		private var _plugin:SomaAssets;
		
		private static var _stage:Stage;
		private static var _configXML:XML;
		private static var _configURL:String;
		
		[BeforeClass]
		public static function runBeforeClass():void {
			_stage = FlexGlobals.topLevelApplication.stage;
			_configURL = "assets/xml/assets.xml";
			_configXML = <loader connections="3">
				<group id="group0" connections="1" preventCache="true">
					<group id="group00" connections="1" preventCache="true">
						<asset id="img0" src="assets/img/image00.jpg" smoothing="true" transparent="true" weight="174015"/>
						<asset id="img1" src="assets/img/image01.jpg" smoothing="true" transparent="true" weight="165158"/>
					</group>
					<asset id="img2" src="assets/img/image02.jpg" fillColor="0xFF0000" smoothing="true" transparent="true" weight="156926"/>
					<asset id="img3" src="assets/img/image03.jpg" fillColor="0xFF0000" smoothing="true" transparent="true" weight="177053"/>
				</group>
				<group id="group1" connections="2" preventCache="true">
					<asset id="swf0" src="assets/swf/flash0.swf" priority="1" weight="1915"/>
					<asset id="swf1" src="assets/swf/flash1.swf" priority="2" weight="2005"/>
				</group>
				<asset id="css" src="assets/css/stylesheet.css" preventCache="true" weight="352"/>
				<asset id="json" src="assets/json/data.json" preventCache="true" weight="582"/>
				<asset id="sound" src="assets/sounds/sample.mp3" preventCache="true" weight="16718"/>
				<asset id="text" src="assets/text/text.txt" preventCache="true" weight="574"/>
				<asset id="xml" src="assets/xml/sample.xml" preventCache="true" weight="79"/>
				<asset id="video" src="assets/video/sample.flv" weight="1550580"/>
				<asset id="zip" src="assets/zip/file.zip" weight="3493"/>
			</loader>;
		}
		
		[AfterClass]
		public static function runAfterClass():void {
			_stage = null;
		} 
		
		[Before]
		public function runBefore():void {
			_soma = new Soma(_stage);
		}
		
		[After]
		public function runAfter():void {
			_plugin.dispose();
			_soma.dispose();
			_plugin = null;
			_soma = null;
		}
		
		private function timeOutHandler(event:SomaAssetsEvent):void {
			fail("time out");
		}
		
		[Test(async)]
		public function testAssetLoaded():void {
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_plugin.addConfig(_configXML);
			_soma.addEventListener(SomaAssetsEvent.ASSET_LOADED, Async.asyncHandler(this, testAssetLoadedSuccess, 500, null, timeOutHandler));
			_plugin.loader.start();
		}
		
		private function testAssetLoadedSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			assertNotNull(event.asset);
			assertNotNull(event.loader);
			assertNotNull(event.config);
			assertNotNull(event.path);
			assertNotNull(event.plugin);
		}
		
		[Test(async)]
		public function testConfigLoadedFromXML():void {
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_soma.addEventListener(SomaAssetsEvent.CONFIG_LOADED, Async.asyncHandler(this, testConfigLoadedFromXMLSuccess, 500, null, timeOutHandler));
			_plugin.addConfig(_configXML);
		}
		
		private function testConfigLoadedFromXMLSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			assertNotNull(event.config);
			assertNotNull(_plugin.config);
		}
		
		[Test(async)]
		public function testConfigLoadedFromString():void {
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_soma.addEventListener(SomaAssetsEvent.CONFIG_LOADED, Async.asyncHandler(this, testConfigLoadedFromStringSuccess, 500, null, timeOutHandler));
			_plugin.addConfig(_configURL);
		}
		
		private function testConfigLoadedFromStringSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			assertNotNull(event.config);
			assertNotNull(_plugin.config);
		}
		
		[Test(async)]
		public function testLoaderReady():void {
			_soma.addEventListener(SomaAssetsEvent.LOADER_READY, Async.asyncHandler(this, testLoaderReadySuccess, 500, null, timeOutHandler));
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
		}
		
		private function testLoaderReadySuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			assertNotNull(event.loader);
		}
		
		[Test(async)]
		public function testLoaderComplete():void {
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_plugin.addConfig(_configXML);
			_plugin.loader.start();
			_soma.addEventListener(SomaAssetsEvent.LOADER_COMPLETE, Async.asyncHandler(this, testLoaderCompleteSuccess, 500, null, timeOutHandler));
		}
		
		private function testLoaderCompleteSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
		}
		
		[Test(async)]
		public function testLoaderError():void {
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_plugin.loader.add("wrong-url", new URLRequest("file-that-does-not-exist.txt"));
			_plugin.loader.start();
			_soma.addEventListener(SomaAssetsEvent.ERROR, Async.asyncHandler(this, testLoaderErrorSuccess, 500, null, timeOutHandler));
		}
		
		private function testLoaderErrorSuccess(event:SomaAssetsEvent, data:Object):void {
			assertNotNull(event.errorMessage);
			assertNotNull(event.errorType);
			data;
		}
		
		[Test(async)]
		public function testLoaderErrorDeeper():void {
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_plugin.addConfig(_configXML);
			IAssetLoader(_plugin.getLoader("group0/group00")).add("wrong-url", new URLRequest("file-that-does-not-exist.txt"));
			_plugin.loader.start();
			_soma.addEventListener(SomaAssetsEvent.ERROR, Async.asyncHandler(this, testLoaderErrorDeeperSuccess, 500, null, timeOutHandler));
		}
		
		private function testLoaderErrorDeeperSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
		}
		
	}
}
