package com.soma.plugins.assets.tests {

	import com.soma.core.Soma;
	import com.soma.core.di.SomaInjector;
	import com.soma.core.interfaces.ISoma;
	import com.soma.plugins.assets.SomaAssets;
	import com.soma.plugins.assets.events.SomaAssetsEvent;
	import com.soma.plugins.assets.tests.support.Injectee;
	import com.soma.plugins.assets.vo.SomaAssetsVO;

	import org.assetloader.core.IAssetLoader;
	import org.assetloader.core.ILoader;
	import org.flexunit.asserts.assertFalse;
	import org.flexunit.asserts.assertNotNull;
	import org.flexunit.asserts.assertTrue;
	import org.flexunit.asserts.fail;
	import org.flexunit.async.Async;

	import mx.core.FlexGlobals;

	import flash.display.Bitmap;
	import flash.display.Stage;
	import flash.text.StyleSheet;
	/**
	 * <b>Author:</b> Romuald Quantin - <a href="http://www.soundstep.com/" target="_blank">www.soundstep.com</a><br />
	 * <b>Class version:</b> 1.0<br />
	 * <b>Actionscript version:</b> 3.0<br />
	 * <b>Date:</b> Oct 6, 2010<br />
	 * @example
	 * <listing version="3.0"></listing>
	 */
	public class InjectorTests {

		
		private var _soma:ISoma;
		private var _plugin:SomaAssets;
		
		private static var _stage:Stage;
		private static var _configXML:XML;
		
		[BeforeClass]
		public static function runBeforeClass():void {
			_stage = FlexGlobals.topLevelApplication.stage;
			_configXML = <loader connections="3">
				<group id="group0" connections="1" preventCache="true">
					<group id="group00" connections="1" preventCache="true">
						<asset id="img0" src="assets/img/image00.jpg" smoothing="true" transparent="true" weight="174015"/>
						<asset id="img1" src="assets/img/image01.jpg" smoothing="true" transparent="true" weight="165158"/>
					</group>
					<asset id="img2" src="assets/img/image02.jpg" fillColor="0xFF0000" smoothing="true" transparent="true" weight="156926"/>
					<asset id="img3" src="assets/img/image03.jpg" fillColor="0xFF0000" smoothing="true" transparent="true" weight="177053"/>
				</group>
				<group id="group1" connections="2" preventCache="true">
					<asset id="swf0" src="assets/swf/flash0.swf" priority="1" weight="1915"/>
					<asset id="swf1" src="assets/swf/flash1.swf" priority="2" weight="2005"/>
				</group>
				<asset id="css" src="assets/css/stylesheet.css" preventCache="true" weight="352"/>
				<asset id="json" src="assets/json/data.json" preventCache="true" weight="582"/>
				<asset id="sound" src="assets/sounds/sample.mp3" preventCache="true" weight="16718"/>
				<asset id="text" src="assets/text/text.txt" preventCache="true" weight="574"/>
				<asset id="xml" src="assets/xml/sample.xml" preventCache="true" weight="79"/>
				<asset id="video" src="assets/video/sample.flv" weight="1550580"/>
				<asset id="zip" src="assets/zip/file.zip" weight="3493"/>
			</loader>;
		}
		
		[AfterClass]
		public static function runAfterClass():void {
			_stage = null;
		} 
		
		[Before]
		public function runBefore():void {
			_soma = new Soma(_stage, SomaInjector);
			_plugin = _soma.createPlugin(SomaAssets, new SomaAssetsVO(_soma)) as SomaAssets;
			_plugin.addConfig(_configXML);
		}
		
		[After]
		public function runAfter():void {
			_plugin.dispose();
			_soma.dispose();
			_plugin = null;
			_soma = null;
		}
		
		[Test]
		public function testInjectionNameGlobal():void {
			assertTrue(_soma.injector.hasMapping(SomaAssets, SomaAssets.INJECTION_NAME));
			assertTrue(_soma.injector.hasMapping(IAssetLoader, SomaAssets.INJECTION_NAME));
			assertTrue(_soma.injector.hasMapping(XML, SomaAssets.INJECTION_NAME));
			_plugin.dispose();
			assertFalse(_soma.injector.hasMapping(SomaAssets, SomaAssets.INJECTION_NAME));
			assertFalse(_soma.injector.hasMapping(IAssetLoader, SomaAssets.INJECTION_NAME));
			assertFalse(_soma.injector.hasMapping(XML, SomaAssets.INJECTION_NAME));
		}
		
		[Test(async)]
		public function testInjectionNameAsset():void {
			_soma.addEventListener(SomaAssetsEvent.LOADER_COMPLETE, Async.asyncHandler(this, testInjectionNameAssetSuccess, 500, null, testInjectionNameAssetFailed));
			_plugin.loader.start();
		}
		
		private function testInjectionNameAssetSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			assertNotNull(_plugin.getAssets("css"));
			assertTrue(_soma.injector.hasMapping(StyleSheet, "css"));
			_plugin.dispose();
			assertFalse(_soma.injector.hasMapping(StyleSheet, "css"));
		}
		
		private function testInjectionNameAssetFailed(event:SomaAssetsEvent):void {
			fail("testInjectionNameAsset failed on time out");
		}
		
		[Test(async)]
		public function testInjectionNameAssetDeeper():void {
			_soma.addEventListener(SomaAssetsEvent.LOADER_COMPLETE, Async.asyncHandler(this, testInjectionNameAssetDeeperSuccess, 500, null, testInjectionNameAssetDeeperFailed));
			_plugin.loader.start();
		}
		
		private function testInjectionNameAssetDeeperSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			var path:String = "group0/group00/img0";
			assertNotNull(_plugin.getAssets(path));
			assertTrue(_soma.injector.hasMapping(Bitmap, path));
			_plugin.dispose();
			assertFalse(_soma.injector.hasMapping(Bitmap, path));
		}
		
		private function testInjectionNameAssetDeeperFailed(event:SomaAssetsEvent):void {
			fail("testInjectionNameAssetDeeper failed on time out");
		}
		
		[Test]
		public function testInjectionNameGroup():void {
			var path:String = "group0";
			assertTrue(_soma.injector.hasMapping(IAssetLoader, path));
			assertTrue(_soma.injector.hasMapping(ILoader, path));
			_plugin.dispose();
			assertFalse(_soma.injector.hasMapping(IAssetLoader, path));
			assertFalse(_soma.injector.hasMapping(ILoader, path));
		}
		
		[Test]
		public function testInjectionNameGroupDeeper():void {
			var path:String = "group0/group00";
			assertTrue(_soma.injector.hasMapping(IAssetLoader, path));
			assertTrue(_soma.injector.hasMapping(ILoader, path));
			_plugin.dispose();
			assertFalse(_soma.injector.hasMapping(IAssetLoader, path));
			assertFalse(_soma.injector.hasMapping(ILoader, path));
		}
		
		[Test(async)]
		public function testInjection():void {
			_soma.addEventListener(SomaAssetsEvent.LOADER_COMPLETE, Async.asyncHandler(this, testInjectionSuccess, 500, null, testInjectionFailed));
			_plugin.loader.start();
		}
		
		private function testInjectionSuccess(event:SomaAssetsEvent, data:Object):void {
			data;
			_soma.injector.createInstance(Injectee);
		}
		
		private function testInjectionFailed(event:SomaAssetsEvent):void {
			fail("testInjection failed on time out");
		}
		
	}
}
